@@ -1,12 +1,15 @@
+import httplib
 import logging
 import cjson
 from urllib2 import urlopen
 from urllib2 import HTTPError
 from urllib2 import Request as URLLibRequest
 from .exceptions import JSONRestRequestException
-from .sentinels import NO_DATA
+from sentinels import Sentinel
 from .raw import Raw
 
+NO_DATA = Sentinel("NO_DATA")
+
 _logger = logging.getLogger("json_rest")
 _logger.addHandler(logging.NullHandler())
 
@@ -20,7 +23,7 @@ class AbstractJSONRestSender(object):
     def delete(self, uri=None):
         return self.send_request('DELETE', uri, NO_DATA)
     def send_request(self, method, uri, data):
-        raise NotImplementedError()
+        raise NotImplementedError() # pragma: no cover
 
 class JSONRestSender(AbstractJSONRestSender):
     def __init__(self, uri):
@@ -40,42 +43,40 @@ class JSONRestSender(AbstractJSONRestSender):
         returned = type(self)(self._uri)
         returned.append_uri_fragment(resource)
         return returned
-    def send_request(self, method, uri, data):
-        if data is NO_DATA:
-            send_data = ''
-        else:
-            send_data = cjson.encode(data)
+    def _create_request(self, method, send_data, uri):
         full_uri = self._uri
         if uri is not None:
             full_uri = _urljoin(full_uri, uri)
+        send_data, content_type = self._get_send_data_and_content_type(send_data)
         request = RestRequest(method, full_uri, send_data)
-        _logger.debug("Sending request: %s", request)
         request.add_header("Accept", "application/json")
-        if send_data:
+        if content_type is not None:
             request.add_header('Content-type', 'application/json')
+        return request
+    def send_request(self, method, uri, data):
+        request = self._create_request(method, data, uri)
+        _logger.debug("Sending request: %s", request)
         try:
             response = urlopen(request)
         except HTTPError, e:
-            error_data = e.fp.read()
+            error_data = self._parse_response_data(e.code, e.fp.read(), e.headers)
             _logger.debug("Got response: code=%s data=%r", e.code, error_data)
-            if e.headers.get('content-type') == 'application/json':
-                error_data = cjson.decode(error_data)
-            else:
-                error_data = Raw(error_data)
-            raise JSONRestRequestException(
-                method, full_uri, e.code,
-                sent_headers=request.headers,
-                sent_data=data, received_headers=e.headers, received_data=error_data)
+            raise JSONRestRequestException.from_request_and_http_error(request, e, data, error_data)
         response_data = response.read()
         _logger.debug("Got response: code=%s data=%r", response.code, response_data)
-        return self._build_response_data(response, response_data)
-    def _build_response_data(self, response, response_data):
-        if response_data:
-            if response.headers.get('Content-type') == 'application/json':
-                return cjson.decode(response_data)
-            else:
-                return Raw(response_data)
-        return NO_DATA
+        return self._parse_response_data(response.code, response_data, response.headers)
+    def _get_send_data_and_content_type(self, send_data):
+        if isinstance(send_data, Raw):
+            return send_data.data, None
+        if send_data is NO_DATA:
+            return None, None
+        return cjson.encode(send_data), "application/json"
+    def _parse_response_data(self, code, response_data, response_headers):
+        if code == httplib.NO_CONTENT:
+            return NO_DATA
+        if response_headers.get('content-type') == 'application/json':
+             return cjson.decode(response_data)
+        return Raw(response_data)
 
 def _urljoin(*urls):
     "Like urlparse's urljoin, only more forgiving for lack of slashes"
